<?php
/*
 * Import in reverse order:
 * Read the pages of an input PDF document and output them in reverse order
 *  
 * Open the pages of the input document in reverse order and place them in
 * the output PDF.
 *  
 * Required software: PDFlib+PDI/PPS 9
 * Required data: PDF document
 */

/* This is where the data files are. Adjust as necessary. */
$searchpath = dirname(__FILE__,3)."/input";

/* By default annotations are also imported. In some cases this
 * requires the Noto fonts for creating annotation appearance streams.
 * We therefore set the searchpath to also point to the font directory.
 */
$fontpath = dirname(__FILE__,3)."/resource/font";
$outfile = "";
$title = "Import in Reverse Order";

$pdffile = "PDFlib-datasheet.pdf";

try {
    $p = new pdflib();

    $p->set_option("searchpath={" . $searchpath . "}");

    $p->set_option("searchpath={" . $fontpath . "}");

    /* This means we must check return values of load_font() etc. */
    $p->set_option("errorpolicy=return");

    if ($p->begin_document($outfile, "") == 0)
        throw new Exception("Error: " . $p->get_errmsg());

    $p->set_info("Creator", "PDFlib Cookbook");
    $p->set_info("Title", $title);

    /* Open the input PDF */
    $indoc = $p->open_pdi_document($pdffile, "");
    if ($indoc == 0)
        throw new Exception("Error: " . $p->get_errmsg());

    /* Retrieve the overall number of pages for the input document */
    $endpage = (int) $p->pcos_get_number($indoc, "length:pages");

    /* Loop over all pages of the input document in reverse order */
    for ($pageno = $endpage; $pageno > 0; $pageno--)
    {
        $page = $p->open_pdi_page($indoc, $pageno, "");

        if ($page == 0)
            throw new Exception("Error: " . $p->get_errmsg());

        /* Page size may be adjusted by fit_pdi_page() */
        $p->begin_page_ext(0, 0, "width=a4.width height=a4.height");

        /* Place the imported page without performing
         * any changes on the output page
         */
        $p->fit_pdi_page($page, 0, 0, "adjustpage");
        
        $p->end_page_ext("");

        $p->close_pdi_page($page);
    }
    $p->end_document("");

    $buf = $p->get_buffer();
    $len = strlen($buf);

    header("Content-type: application/pdf");
    header("Content-Length: $len");
    header("Content-Disposition: inline; filename=import_in_reverse_order.pdf");
    print $buf;


}

catch (PDFlibException $e) {
    echo("PDFlib exception occurred in sample:\n" .
        "[" . $e->get_errnum() . "] " . $e->get_apiname() . ": " .
        $e->get_errmsg() . "\n");
    exit(1);
}
catch (Throwable $e) {
    echo($e);
    exit(1);
}

$p = 0;

?>
